@@ -1,20 +1,19 @@
-/* Copyright (C) 2019 Greenbone Networks GmbH
+/* Copyright (C) 2019-2020 Greenbone Networks GmbH
  *
- * SPDX-License-Identifier: GPL-2.0-or-later
+ * SPDX-License-Identifier: AGPL-3.0-or-later
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -883,7 +882,7 @@ tls_certificate_origin_extra_xml (const char *origin_type,
       report = 0;
       if (find_report_with_permission (origin_id, &report, "get_reports"))
         {
-          g_warning ("%s : error getting report", __FUNCTION__);
+          g_warning ("%s : error getting report", __func__);
         }
 
       if (report)
@@ -899,7 +898,7 @@ tls_certificate_origin_extra_xml (const char *origin_type,
           task = 0;
           if (report_task (report, &task))
             {
-              g_warning ("%s : error getting report task", __FUNCTION__);
+              g_warning ("%s : error getting report task", __func__);
             }
 
           if (task)
