@@ -1,20 +1,19 @@
-/* Copyright (C) 2013-2018 Greenbone Networks GmbH
+/* Copyright (C) 2013-2021 Greenbone Networks GmbH
  *
- * SPDX-License-Identifier: GPL-2.0-or-later
+ * SPDX-License-Identifier: AGPL-3.0-or-later
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -41,28 +40,22 @@
 #define G_LOG_DOMAIN "md manage"
 
 /**
- * @brief Test whether a user may perform an operation.
+ * @brief Test whether the current user may perform an operation.
+ *
+ * Does not check if the user is special.
  *
  * @param[in]  operation  Name of operation.
  *
  * @return 1 if user has permission, else 0.
  */
-int
-acl_user_may (const char *operation)
+static int
+user_may_internal (const char *operation)
 {
   int ret;
   gchar *quoted_operation;
 
   assert (operation);
 
-  if (strlen (current_credentials.uuid) == 0)
-    /* Allow the dummy user in init_manage to do anything. */
-    return 1;
-
-  if (sql_int ("SELECT user_can_everything ('%s');",
-               current_credentials.uuid))
-    return 1;
-
   quoted_operation = sql_quote (operation);
 
   ret = sql_int (ACL_USER_MAY ("0"),
@@ -79,6 +72,103 @@ acl_user_may (const char *operation)
   return ret;
 }
 
+/**
+ * @brief Check if a string array contains a string, ignoring case.
+ *
+ * @param[in]  strv    String array.
+ * @param[in]  string  String.
+ *
+ * @return 1 if strv contains string, else 0.
+ */
+static int
+strv_case_eq (gchar **strv, const gchar *string)
+{
+  if (string == NULL)
+    return 0;
+
+  while (*strv)
+    if (strcasecmp (*strv, string) == 0)
+      return 1;
+    else
+      strv++;
+
+  return 0;
+}
+
+/**
+ * @brief Get commands that the current user may run.
+ *
+ * @param[in]  disabled_commands  All disabled commands.
+ *
+ * @return Freshly allocated list of commands.  Free with g_free.
+ */
+command_t *
+acl_commands (gchar **disabled_commands)
+{
+  command_t *all, *commands;
+  int index, length, special_user;
+
+  /* Count maximum number of commands. */
+
+  length = 1;
+  all = gmp_commands;
+  while ((*all).name)
+    {
+      length++;
+      all++;
+    }
+
+  /* Fill return array with allowed commands. */
+
+  special_user = ((current_credentials.uuid == NULL)
+                  || (strlen (current_credentials.uuid) == 0));
+
+  if (special_user == 0)
+    special_user = sql_int ("SELECT user_can_everything ('%s');",
+                            current_credentials.uuid);
+
+  commands = g_malloc0 (length * sizeof (*commands));
+  all = gmp_commands;
+  index = 0;
+  while ((*all).name)
+    {
+      if ((disabled_commands == NULL
+           || strv_case_eq (disabled_commands, (*all).name)
+              == 0)
+          && (special_user
+              || user_may_internal ((*all).name)))
+        {
+          commands[index].name = (*all).name;
+          commands[index].summary = (*all).summary;
+          index++;
+        }
+      all++;
+    }
+
+  return commands;
+}
+
+/**
+ * @brief Test whether a user may perform an operation.
+ *
+ * @param[in]  operation  Name of operation.
+ *
+ * @return 1 if user has permission, else 0.
+ */
+int
+acl_user_may (const char *operation)
+{
+  if (strlen (current_credentials.uuid) == 0)
+    /* Allow the dummy user in init_manage to do anything. */
+    return 1;
+
+  if (sql_int ("SELECT user_can_everything ('%s');",
+               current_credentials.uuid))
+    return 1;
+
+  return user_may_internal (operation);
+}
+
 /**
  * @brief Check whether a role has Super Admin capability.
  *
@@ -905,6 +995,8 @@ acl_user_has_access_uuid (const char *type, const char *uuid,
  * @param[in]  owner_filter    Owner filter keyword.
  * @param[in]  resource        Resource.
  * @param[in]  permissions     Permissions.
+ * @param[in]  with_optional   Whether the WITH clause is optional.
+ * @param[in]  with_prefix     Optional prefix for WITH subqueries.
  * @param[out] with            Address for WITH clause if allowed, else NULL.
  *
  * @return Newly allocated owned clause.
@@ -913,7 +1005,8 @@ static gchar *
 acl_where_owned_user (const char *user_id, const char *user_sql,
                       const char *type, const get_data_t *get, int owned,
                       const gchar *owner_filter, resource_t resource,
-                      array_t *permissions, gchar **with)
+                      array_t *permissions, int with_optional,
+                      const char *with_prefix, gchar **with)
 {
   gchar *owned_clause, *filter_owned_clause;
   GString *permission_or;
@@ -921,10 +1014,66 @@ acl_where_owned_user (const char *user_id, const char *user_sql,
   guint index;
 
   if (with)
-    *with = NULL;
+    {
+      *with = g_strdup_printf
+               ("WITH %spermissions_subject"
+                "     AS (SELECT * FROM permissions"
+                "         WHERE subject_location"
+                "               = " G_STRINGIFY (LOCATION_TABLE)
+                "         AND ((subject_type = 'user'"
+                "               AND subject"
+                "                   = (%s))"
+                "              OR (subject_type = 'group'"
+                "                  AND subject"
+                "                      IN (SELECT DISTINCT \"group\""
+                "                          FROM group_users"
+                "                          WHERE \"user\""
+                "                                = (%s)))"
+                "              OR (subject_type = 'role'"
+                "                  AND subject"
+                "                      IN (SELECT DISTINCT role"
+                "                          FROM role_users"
+                "                          WHERE \"user\""
+                "                                = (%s))))),"
+                "     %ssuper_on_users"
+                "     AS (SELECT DISTINCT *"
+                "         FROM (SELECT resource FROM %spermissions_subject"
+                "               WHERE name = 'Super'"
+                "               AND resource_type = 'user'"
+                "               UNION"
+                "               SELECT \"user\" FROM role_users"
+                "               WHERE role"
+                "                     IN (SELECT resource"
+                "                         FROM %spermissions_subject"
+                "                         WHERE name = 'Super'"
+                "                         AND resource_type = 'role')"
+                "               UNION"
+                "               SELECT \"user\" FROM group_users"
+                "               WHERE \"group\""
+                "                     IN (SELECT resource"
+                "                         FROM %spermissions_subject"
+                "                         WHERE name = 'Super'"
+                "                         AND resource_type = 'group'))"
+                "              AS all_users)",
+                with_prefix ? with_prefix : "",
+                user_sql,
+                user_sql,
+                user_sql,
+                with_prefix ? with_prefix : "",
+                with_prefix ? with_prefix : "",
+                with_prefix ? with_prefix : "",
+                with_prefix ? with_prefix : "");
+    }
 
   if (owned == 0)
-   return g_strdup (" t ()");
+    {
+      if (with_optional && with)
+        {
+          g_free (*with);
+          *with = NULL;
+        }
+      return g_strdup (" t ()");
+    }
 
   permission_or = g_string_new ("");
   index = 0;
@@ -965,61 +1114,18 @@ acl_where_owned_user (const char *user_id, const char *user_sql,
 
       /* Caller supports WITH clause. */
 
-      *with = g_strdup_printf
-               ("WITH permissions_subject"
-                "     AS (SELECT * FROM permissions"
-                "         WHERE subject_location"
-                "               = " G_STRINGIFY (LOCATION_TABLE)
-                "         AND ((subject_type = 'user'"
-                "               AND subject"
-                "                   = (%s))"
-                "              OR (subject_type = 'group'"
-                "                  AND subject"
-                "                      IN (SELECT DISTINCT \"group\""
-                "                          FROM group_users"
-                "                          WHERE \"user\""
-                "                                = (%s)))"
-                "              OR (subject_type = 'role'"
-                "                  AND subject"
-                "                      IN (SELECT DISTINCT role"
-                "                          FROM role_users"
-                "                          WHERE \"user\""
-                "                                = (%s))))),"
-                "     super_on_users"
-                "     AS (SELECT DISTINCT *"
-                "         FROM (SELECT resource FROM permissions_subject"
-                "               WHERE name = 'Super'"
-                "               AND resource_type = 'user'"
-                "               UNION"
-                "               SELECT \"user\" FROM role_users"
-                "               WHERE role"
-                "                     IN (SELECT resource"
-                "                         FROM permissions_subject"
-                "                         WHERE name = 'Super'"
-                "                         AND resource_type = 'role')"
-                "               UNION"
-                "               SELECT \"user\" FROM group_users"
-                "               WHERE \"group\""
-                "                     IN (SELECT resource"
-                "                         FROM permissions_subject"
-                "                         WHERE name = 'Super'"
-                "                         AND resource_type = 'group'))"
-                "              AS all_users)",
-                user_sql,
-                user_sql,
-                user_sql);
-
       permission_clause = NULL;
       if (user_id && index)
         {
           gchar *clause;
           clause
            = g_strdup_printf ("OR EXISTS"
-                              " (SELECT id FROM permissions_subject"
+                              " (SELECT id FROM %spermissions_subject"
                               "  WHERE resource = %ss%s.id"
                               "  AND resource_type = '%s'"
                               "  AND resource_location = %i"
                               "  AND (%s))",
+                              with_prefix ? with_prefix : "",
                               type,
                               get->trash && strcmp (type, "task") ? "_trash" : "",
                               type,
@@ -1030,22 +1136,24 @@ acl_where_owned_user (const char *user_id, const char *user_sql,
             permission_clause
              = g_strdup_printf ("%s"
                                 " OR EXISTS"
-                                " (SELECT id FROM permissions_subject"
+                                " (SELECT id FROM %spermissions_subject"
                                 "  WHERE resource = reports%s.task"
                                 "  AND resource_type = 'task'"
                                 "  AND (%s))",
                                 clause,
+                                with_prefix ? with_prefix : "",
                                 get->trash ? "_trash" : "",
                                 permission_or->str);
           else if (strcmp (type, "result") == 0)
             permission_clause
              = g_strdup_printf ("%s"
                                 " OR EXISTS"
-                                " (SELECT id FROM permissions_subject"
+                                " (SELECT id FROM %spermissions_subject"
                                 "  WHERE resource = results%s.task"
                                 "  AND resource_type = 'task'"
                                 "  AND (%s))",
                                 clause,
+                                with_prefix ? with_prefix : "",
                                 get->trash ? "_trash" : "",
                                 permission_or->str);
 
@@ -1094,13 +1202,13 @@ acl_where_owned_user (const char *user_id, const char *user_sql,
                               "                   FROM role_users"
                               "                   WHERE \"user\" = (%s))))"
                               /* Or the user has super permission on all. */
-                              "  OR EXISTS (SELECT * FROM permissions_subject"
+                              "  OR EXISTS (SELECT * FROM %spermissions_subject"
                               "             WHERE name = 'Super'"
                               "             AND (resource = 0))"
                               /* Or the user has super permission on the owner,
                                * (directly, via the role, or via the group). */
-                              "  OR permissions%s.owner IN (SELECT *"
-                              "                            FROM super_on_users)"
+                              "  OR permissions%s.owner IN"
+                              "      (SELECT * FROM %ssuper_on_users)"
                               "  %s)",
                               get->trash ? "_trash" : "",
                               user_sql,
@@ -1124,7 +1232,9 @@ acl_where_owned_user (const char *user_id, const char *user_sql,
                               table_trash ? "_trash" : "",
                               table_trash ? "_trash" : "",
                               user_sql,
+                              with_prefix ? with_prefix : "",
                               table_trash ? "_trash" : "",
+                              with_prefix ? with_prefix : "",
                               permission_clause ? permission_clause : "");
         }
       else
@@ -1134,19 +1244,21 @@ acl_where_owned_user (const char *user_id, const char *user_sql,
                             " ((%ss%s.owner"
                             "   = (%s))"
                             /* Or the user has super permission on all. */
-                            "  OR EXISTS (SELECT * FROM permissions_subject"
+                            "  OR EXISTS (SELECT * FROM %spermissions_subject"
                             "             WHERE name = 'Super'"
                             "             AND (resource = 0))"
                             /* Or the user has super permission on the owner,
                              * (directly, via the role, or via the group). */
                             "  OR %ss%s.owner IN (SELECT *"
-                            "                     FROM super_on_users)"
+                            "                     FROM %ssuper_on_users)"
                             "  %s)",
                             type,
                             table_trash ? "_trash" : "",
                             user_sql,
+                            with_prefix ? with_prefix : "",
                             type,
                             table_trash ? "_trash" : "",
+                            with_prefix ? with_prefix : "",
                             permission_clause ? permission_clause : "");
 
       g_free (permission_clause);
@@ -1379,6 +1491,7 @@ acl_where_owned_user (const char *user_id, const char *user_sql,
  * @param[in]  owner_filter    Owner filter keyword.
  * @param[in]  resource        Resource.
  * @param[in]  permissions     Permissions.
+ * @param[in]  with_optional   Whether permissions WITH clauses are optional.
  * @param[out] with            Address for WITH clause if allowed, else NULL.
  *
  * @return Newly allocated owned clause.
@@ -1386,7 +1499,7 @@ acl_where_owned_user (const char *user_id, const char *user_sql,
 gchar *
 acl_where_owned (const char *type, const get_data_t *get, int owned,
                  const gchar *owner_filter, resource_t resource,
-                 array_t *permissions, gchar **with)
+                 array_t *permissions, int with_optional, gchar **with)
 {
   gchar *ret, *user_sql;
   if (current_credentials.uuid)
@@ -1396,7 +1509,7 @@ acl_where_owned (const char *type, const get_data_t *get, int owned,
     user_sql = NULL;
   ret = acl_where_owned_user (current_credentials.uuid, user_sql, type, get,
                               owned, owner_filter, resource, permissions,
-                              with);
+                              with_optional, NULL, with);
   g_free (user_sql);
   return ret;
 }
@@ -1407,13 +1520,15 @@ acl_where_owned (const char *type, const get_data_t *get, int owned,
  * @param[in]  type      Type of resource.
  * @param[in]  user_sql  SQL for getting user.  If NULL SQL will be for current
  *                       user.
+ * @param[in]  with_prefix  Optional prefix for WITH clause.
  * @param[out] with      Return location for WITH preselection clause if
  *                       desired, else NULL.
  *
  * @return Newly allocated owned clause.
  */
 gchar *
-acl_where_owned_for_get (const char *type, const char *user_sql, gchar **with)
+acl_where_owned_for_get (const char *type, const char *user_sql, 
+                         const char *with_prefix, gchar **with)
 {
   gchar *owned_clause;
   get_data_t get;
@@ -1442,6 +1557,8 @@ acl_where_owned_for_get (const char *type, const char *user_sql, gchar **with)
                                        "any",
                                        0,              /* Resource. */
                                        permissions,
+                                       0,              /* WITH not optional */
+                                       with_prefix,
                                        with);
   array_free (permissions);
   g_free (user_sql_new);
