@@ -1,20 +1,19 @@
-/* Copyright (C) 2013-2019 Greenbone Networks GmbH
+/* Copyright (C) 2013-2021 Greenbone Networks GmbH
  *
- * SPDX-License-Identifier: GPL-2.0-or-later
+ * SPDX-License-Identifier: AGPL-3.0-or-later
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -852,9 +851,9 @@ make_tls_certificate_214 (user_t owner,
   quoted_certificate_b64
     = certificate_b64 ? sql_quote (certificate_b64) : NULL;
   quoted_subject_dn
-    = subject_dn ? sql_quote (subject_dn) : NULL;
+    = subject_dn ? sql_ascii_escape_and_quote (subject_dn) : NULL;
   quoted_issuer_dn
-    = issuer_dn ? sql_quote (issuer_dn) : NULL;
+    = issuer_dn ? sql_ascii_escape_and_quote (issuer_dn) : NULL;
   quoted_md5_fingerprint
     = md5_fingerprint ? sql_quote (md5_fingerprint) : NULL;
   quoted_sha256_fingerprint
@@ -1119,7 +1118,7 @@ migrate_213_to_214 ()
                               &serial);
           else
             g_warning ("%s: No SSLDetails found for fingerprint %s",
-                       __FUNCTION__,
+                       __func__,
                        scanner_fpr);
 
           free (ssldetails);
@@ -1484,7 +1483,7 @@ replace_preference_names_219_to_220 (const char *table_name)
           g_free (quoted_new_name);
         }
       else
-        g_warning ("%s: No new name for '%s'", __FUNCTION__, old_name);
+        g_warning ("%s: No new name for '%s'", __func__, old_name);
     }
   cleanup_iterator (&preferences);
 }
@@ -1573,7 +1572,9 @@ convert_schedules_221 (gboolean trash)
       ical_string = iterator_string (&schedules, 1);
       schedule_id = iterator_string (&schedules, 2);
 
-      ical_component = icalendar_from_string (ical_string, &error_out);
+      ical_component = icalendar_from_string (ical_string,
+                                              icaltimezone_get_utc_timezone (),
+                                              &error_out);
       if (ical_component == NULL)
         g_warning ("Error converting schedule %s: %s", schedule_id, error_out);
       else
@@ -1630,6 +1631,1026 @@ migrate_220_to_221 ()
   return 0;
 }
 
+/**
+ * @brief Migrate the database from version 221 to version 222.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_221_to_222 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 221. */
+
+  if (manage_db_version () != 221)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* Remove permissions on the remove command MODIFY_REPORT */
+  sql ("DELETE FROM permissions WHERE name = 'modify_report';");
+  sql ("DELETE FROM permissions_trash WHERE name = 'modify_report';");
+
+  /* Set the database version to 222. */
+
+  set_db_version (222);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 222 to version 223.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_222_to_223 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 222. */
+
+  if (manage_db_version () != 222)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* Extend table "nvts" with additional column "solution_method" */
+  sql ("ALTER TABLE IF EXISTS nvts ADD COLUMN solution_method text;");
+
+  /* Set the database version to 223. */
+
+  set_db_version (223);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 223 to version 224.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_223_to_224 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 223. */
+
+  if (manage_db_version () != 223)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* Agents were removed entirely. */
+
+  sql ("DELETE FROM tag_resources WHERE resource_type = 'agent';");
+  sql ("DELETE FROM tag_resources_trash WHERE resource_type = 'agent';");
+
+  sql ("DELETE FROM tags WHERE resource_type = 'agent';");
+  sql ("DELETE FROM tags_trash WHERE resource_type = 'agent';");
+
+  sql ("DELETE FROM permissions WHERE resource_type = 'agent';");
+  sql ("DELETE FROM permissions_trash WHERE resource_type = 'agent';");
+
+  sql ("UPDATE alerts SET filter = 0"
+       " WHERE filter IN (SELECT id FROM filters"
+       "                  WHERE type = 'agent');");
+  sql ("UPDATE alerts_trash SET filter = 0"
+       " WHERE filter IN (SELECT id FROM filters"
+       "                  WHERE type = 'agent')"
+       " AND filter_location = %i;",
+       LOCATION_TABLE);
+  sql ("UPDATE alerts_trash SET filter = 0"
+       " WHERE filter IN (SELECT id FROM filters_trash"
+       "                  WHERE type = 'agent')"
+       " AND filter_location = %i;",
+       LOCATION_TRASH);
+
+  sql ("DELETE FROM filters WHERE type = 'agent';");
+  sql ("DELETE FROM filters_trash WHERE type = 'agent';");
+
+  /* Setting 'Agents Filter'. */
+  sql ("DELETE FROM settings WHERE uuid = '4a1334c1-cb93-4a79-8634-103b0a50bdcd';");
+
+  sql ("DROP TABLE IF EXISTS agents;");
+  sql ("DROP TABLE IF EXISTS agents_trash;");
+
+  /* Set the database version to 224. */
+
+  set_db_version (224);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 224 to version 225.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_224_to_225 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 224. */
+
+  if (manage_db_version () != 224)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* GMP command COMMANDS was removed. */
+
+  sql ("DELETE FROM permissions WHERE name = 'commands';");
+  sql ("DELETE FROM permissions_trash WHERE name = 'commands';");
+
+  /* Set the database version to 225. */
+
+  set_db_version (225);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 225 to version 226.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_225_to_226 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 225. */
+
+  if (manage_db_version () != 225)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* A setting's UUID was changed to the correct length. */
+
+  sql ("UPDATE settings"
+       " SET uuid = 'ce7b121-c609-47b0-ab57-fd020a0336f4a'"
+       " WHERE uuid = 'ce7b121-c609-47b0-ab57-fd020a0336f4';");
+
+  /* Set the database version to 226. */
+
+  set_db_version (226);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 226 to version 227.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_226_to_227 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 226. */
+
+  if (manage_db_version () != 226)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* The unused table port_names was removed. */
+
+  sql ("DROP TABLE port_names;");
+
+  /* Set the database version to 227. */
+
+  set_db_version (227);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Delete results for migrate_227_to_228.
+ *
+ * @param[in]  table  Name of table.
+ *
+ * @return Count of deleted rows.
+ */
+static int
+migrate_227_to_228_delete (const char *table)
+{
+  int location;
+
+  if (strcmp (table, "results") == 0)
+    location = LOCATION_TABLE;
+  else
+    location = LOCATION_TRASH;
+
+  return sql_int (/* Remove results, storing ids. */
+                  "WITH deleted"
+                  " AS (DELETE FROM %s"
+                  "     WHERE EXISTS (SELECT *"
+                  "                   FROM report_host_details, report_hosts"
+                  "                   WHERE report_host_details.report_host"
+                  "                         = report_hosts.id"
+                  "                   AND report_hosts.report = %s.report"
+                  "                   AND report_hosts.host = %s.host"
+                  "                   AND name = 'Host dead'"
+                  "                   AND value = '1')"
+                  "     RETURNING id),"
+                  /* Remove references to results in any tags. */
+                  " dummy1"
+                  " AS (DELETE FROM tag_resources"
+                  "     WHERE resource_type = 'result'"
+                  "     AND resource_location = %i"
+                  "     AND resource IN (SELECT id FROM deleted)),"
+                  /* Remove references to results in any trash tags. */
+                  " dummy2"
+                  " AS (DELETE FROM tag_resources_trash"
+                  "     WHERE resource_type = 'result'"
+                  "     AND resource_location = %i"
+                  "     AND resource IN (SELECT id FROM deleted))"
+                  /* Return count of deleted results. */
+                  " SELECT count(*) from deleted;",
+                  table,
+                  table,
+                  table,
+                  location,
+                  location);
+}
+
+/**
+ * @brief Migrate the database from version 227 to version 228.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_227_to_228 ()
+{
+  int count;
+
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 227. */
+
+  if (manage_db_version () != 227)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* Dead hosts are no longer stored. */
+
+  count = migrate_227_to_228_delete ("results");
+  if (count)
+    g_info ("%s: deleted %i result%s of dead report hosts",
+            __func__,
+            count,
+            count > 1 ? "s" : "");
+
+  count = migrate_227_to_228_delete ("results_trash");
+  if (count)
+    g_info ("%s: deleted %i trashcan result%s of dead report hosts",
+            __func__,
+            count,
+            count > 1 ? "s" : "");
+
+  count = sql_int (/* Delete "Host dead" details, getting dead report_hosts. */
+                   "WITH dead_report_hosts"
+                   " AS (DELETE FROM report_host_details"
+                   "     WHERE name = 'Host dead'"
+                   "     AND value = '1'"
+                   "     RETURNING report_host),"
+                   /* Delete any other details on the dead report_hosts. */
+                   " dummy1"
+                   " AS (DELETE FROM report_host_details"
+                   "     WHERE report_host"
+                   "           IN (SELECT distinct report_host"
+                   "               FROM dead_report_hosts)),"
+                   /* Delete dead report_hosts. */
+                   " deleted"
+                   " AS (DELETE FROM report_hosts"
+                   "     WHERE id IN (SELECT distinct report_host"
+                   "                  FROM dead_report_hosts)"
+                   "     RETURNING report),"
+                   /* Clear report counts for affected reports. */
+                   " dummy2"
+                   " AS (DELETE FROM report_counts"
+                   "     WHERE report IN (SELECT distinct report"
+                   "                      FROM deleted))"
+                   /* Return count of dead report_hosts. */
+                   " SELECT count(*) from deleted;");
+  if (count)
+    g_info ("%s: deleted %i dead report host%s",
+            __func__,
+            count,
+            count > 1 ? "s" : "");
+
+  /* Set the database version to 228. */
+
+  set_db_version (228);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 228 to version 229.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_228_to_229 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 228. */
+
+  if (manage_db_version () != 228)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* Setting UUIDs now have to be unique per owner. */
+  sql ("DELETE FROM settings"
+       " WHERE id NOT IN (SELECT max(id) FROM settings"
+       "                  GROUP BY uuid, owner);");
+
+  sql ("ALTER TABLE settings ADD UNIQUE(uuid, owner);");
+
+  /* Set the database version to 229. */
+
+  set_db_version (229);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 229 to version 230.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_229_to_230 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 229. */
+
+  if (manage_db_version () != 229)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  sql ("ALTER TABLE schedules DROP COLUMN initial_offset;");
+
+  /* Set the database version to 230. */
+
+  set_db_version (230);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Add timezones to schedule iCalendar strings.
+ *
+ * @param[in]  trash  Whether to convert the trash table.
+ */
+static void
+convert_schedules_231 (gboolean trash)
+{
+  iterator_t schedules;
+
+  init_iterator (&schedules,
+                 "SELECT id, icalendar, uuid, timezone FROM %s;",
+                 trash ? "schedules_trash" : "schedules");
+
+  while (next (&schedules))
+    {
+      schedule_t schedule;
+      const char *ical_string, *schedule_id, *zone;
+      icalcomponent *ical_component;
+      icaltimezone *ical_zone;
+      gchar *error_out;
+
+      error_out = NULL;
+      schedule = iterator_int64 (&schedules, 0);
+      ical_string = iterator_string (&schedules, 1);
+      schedule_id = iterator_string (&schedules, 2);
+      zone = iterator_string (&schedules, 3);
+
+      ical_zone = icalendar_timezone_from_string (zone);
+      if (ical_zone == NULL)
+        {
+          g_warning ("%s: error converting schedule %s: timezone '%s'",
+                     __func__, schedule_id, zone);
+          continue;
+        }
+
+      ical_component = icalendar_from_string (ical_string,
+                                              ical_zone,
+                                              &error_out);
+      if (ical_component == NULL)
+        g_warning ("%s: error converting schedule %s: %s", __func__,
+                   schedule_id, error_out);
+      else
+        {
+          gchar *quoted_ical;
+
+          quoted_ical
+            = sql_quote (icalcomponent_as_ical_string (ical_component));
+
+          sql ("UPDATE %s SET icalendar = '%s' WHERE id = %llu",
+               trash ? "schedules_trash" : "schedules",
+               quoted_ical,
+               schedule);
+
+          g_free (quoted_ical);
+        }
+
+      g_free (error_out);
+    }
+
+  cleanup_iterator (&schedules);
+}
+
+/**
+ * @brief Migrate the database from version 230 to version 231.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_230_to_231 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 230. */
+
+  if (manage_db_version () != 230)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* Add timezones to schedule iCalendar strings. */
+  convert_schedules_231 (FALSE);
+  convert_schedules_231 (TRUE);
+
+  /* Set the database version to 231. */
+
+  set_db_version (231);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 231 to version 232.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_231_to_232 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 231. */
+
+  if (manage_db_version () != 231)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* Add path field to results and results_trash. */
+  sql ("ALTER TABLE results ADD COLUMN path text;");
+  sql ("ALTER TABLE results_trash ADD COLUMN path text;");
+
+  /* Set path to empty string */
+  sql ("UPDATE results SET path = '';");
+  sql ("UPDATE results_trash SET path = '';");
+
+  /* Set the database version to 231. */
+
+  set_db_version (232);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Set predefined.
+ *
+ * @param[in]  type   Type to update.
+ * @param[in]  table  Table to update.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_232_to_233_set_predefined (const gchar *type, const gchar *table)
+{
+  GError *error;
+  GDir *dir;
+  const gchar *xml_path;
+  gchar *dir_path;
+
+  dir_path = g_build_filename (GVMD_FEED_DIR,
+                               GMP_VERSION_FEED,
+                               type,
+                               NULL);
+
+  /* Open feed import directory. */
+
+  error = NULL;
+  dir = g_dir_open (dir_path, 0, &error);
+  if (dir == NULL)
+    {
+      g_warning ("%s: Failed to open directory '%s': %s",
+                 __func__, dir_path, error->message);
+      g_error_free (error);
+      g_free (dir_path);
+      return -1;
+    }
+  g_free (dir_path);
+
+  /* Update for each file. */
+
+  while ((xml_path = g_dir_read_name (dir)))
+    if (g_str_has_prefix (xml_path, ".") == 0
+        && strlen (xml_path) >= (36 /* UUID */ + strlen (".xml"))
+        && g_str_has_suffix (xml_path, ".xml"))
+      {
+        gchar *quoted_uuid, *uuid;
+
+        uuid = g_strndup (xml_path + strlen (xml_path) - 4 - 36, 36);
+        quoted_uuid = sql_quote (uuid);
+        g_free (uuid);
+        sql ("UPDATE %s SET predefined = 1 WHERE uuid = '%s';",
+             table, quoted_uuid);
+        g_free (quoted_uuid);
+      }
+
+  /* Cleanup. */
+
+  g_dir_close (dir);
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 232 to version 233.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_232_to_233 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 232. */
+
+  if (manage_db_version () != 232)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* Predefined flag moved to tables. */
+
+  sql ("ALTER TABLE report_formats ADD COLUMN predefined integer;");
+  sql ("ALTER TABLE report_formats_trash ADD COLUMN predefined integer;");
+  sql ("ALTER TABLE port_lists ADD COLUMN predefined integer;");
+  sql ("ALTER TABLE port_lists_trash ADD COLUMN predefined integer;");
+  sql ("ALTER TABLE configs ADD COLUMN predefined integer;");
+  sql ("ALTER TABLE configs_trash ADD COLUMN predefined integer;");
+
+  sql ("UPDATE report_formats SET predefined = 0;");
+  sql ("UPDATE report_formats_trash SET predefined = 0;");
+  sql ("UPDATE port_lists SET predefined = 0;");
+  sql ("UPDATE port_lists_trash SET predefined = 0;");
+  sql ("UPDATE configs SET predefined = 0;");
+  sql ("UPDATE configs_trash SET predefined = 0;");
+
+  sql ("UPDATE report_formats"
+       " SET predefined = 1"
+       " WHERE id IN (SELECT resource FROM resources_predefined"
+       "              WHERE resource_type = 'report_format');");
+
+  migrate_232_to_233_set_predefined ("report_formats", "report_formats");
+  migrate_232_to_233_set_predefined ("configs", "configs");
+  migrate_232_to_233_set_predefined ("port_lists", "port_lists");
+
+  migrate_232_to_233_set_predefined ("report_formats", "report_formats_trash");
+  migrate_232_to_233_set_predefined ("configs", "configs_trash");
+  migrate_232_to_233_set_predefined ("port_lists", "port_lists_trash");
+
+  sql ("DROP TABLE resources_predefined;");
+
+  /* Set the database version to 232. */
+
+  set_db_version (233);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 233 to version 234.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_233_to_234 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 233. */
+
+  if (manage_db_version () != 233)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* Support for GMP Scanners was removed, including setting "GMP Slave
+   * Check Period" and various report slave columns. */
+
+  /* Delete setting. */
+  sql ("DELETE FROM settings WHERE uuid = '63adb79a-62ae-11e9-91ba-28d24461215b';");
+
+  /* Drop columns. */
+  sql ("ALTER TABLE reports"
+       " DROP column slave_task_uuid,"
+       " DROP column slave_uuid,"
+       " DROP column slave_name,"
+       " DROP column slave_host,"
+       " DROP column slave_port;");
+
+  /* Convert existing GMP Scanners to OSP Sensors. */
+  sql ("UPDATE scanners SET type = 5 WHERE type = 4;");
+  sql ("UPDATE scanners_trash SET type = 5 WHERE type = 4;");
+
+  /* Set the database version to 234. */
+
+  set_db_version (234);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 234 to version 235.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_234_to_235 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 234. */
+
+  if (manage_db_version () != 234)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* Support of multiple individual selectable severity classification ranges
+   * was removed. Therefore any entry in settings table where "Severity Class"
+   * is configured, can be removed. This setting has a specific uuid. */
+
+  /* Delete any setting for "Severity Class" . */
+  sql ("DELETE FROM settings WHERE uuid = 'f16bb236-a32d-4cd5-a880-e0fcf2599f59';");
+
+  /* Set the database version to 235. */
+
+  set_db_version (235);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 235 to version 236.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_235_to_236 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 235. */
+
+  if (manage_db_version () != 235)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* Replace any result type "Debug Message" by "Error Message". */
+
+  sql ("UPDATE results SET type = 'Error Message' WHERE type = 'Debug Message';");
+  sql ("UPDATE results_trash SET type = 'Error Message' WHERE type = 'Debug Message';");
+
+  /* Set the database version to 236. */
+
+  set_db_version (236);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 236 to version 237.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_236_to_237 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 236. */
+
+  if (manage_db_version () != 236)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* This previously added a "score" column to the nvts table */
+
+  /* Set the database version to 237. */
+
+  set_db_version (237);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 237 to version 238.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_237_to_238 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 237. */
+
+  if (manage_db_version () != 237)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* This previously added a "score" column to the results and results_trash
+   * tables. */
+
+  /* Set the database version to 238. */
+
+  set_db_version (238);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 238 to version 239.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_238_to_239 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 238. */
+
+  if (manage_db_version () != 238)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* Table targets(_trash) got a allow_simult_ips_same_host column. */
+
+  sql ("ALTER TABLE targets ADD COLUMN"
+       " allow_simult_ips_same_host integer DEFAULT 1;");
+
+  sql ("ALTER TABLE targets_trash ADD COLUMN"
+       " allow_simult_ips_same_host integer DEFAULT 1;");
+
+  /* Set the database version to 239. */
+
+  set_db_version (239);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 239 to version 240.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_239_to_240 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 239. */
+
+  if (manage_db_version () != 239)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* Table results also got a score column, for extended severities. */
+
+  sql ("DROP VIEW IF EXISTS vulns;");
+
+  /* Set the database version to 240. */
+
+  set_db_version (240);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 240 to version 241.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_240_to_241 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 240. */
+
+  if (manage_db_version () != 240)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* Rename allow_simult_ips_same_host column to allow_simultaneous_ips */
+
+  sql ("ALTER TABLE targets RENAME COLUMN"
+       " allow_simult_ips_same_host TO allow_simultaneous_ips;");
+
+  sql ("ALTER TABLE targets_trash RENAME COLUMN"
+       " allow_simult_ips_same_host TO allow_simultaneous_ips;");
+
+  /* Set the database version to 241. */
+
+  set_db_version (241);
+
+  sql_commit ();
+
+  return 0;
+}
+
+/**
+ * @brief Migrate the database from version 241 to version 242.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+migrate_241_to_242 ()
+{
+  sql_begin_immediate ();
+
+  /* Ensure that the database is currently version 241. */
+
+  if (manage_db_version () != 241)
+    {
+      sql_rollback ();
+      return -1;
+    }
+
+  /* Update the database. */
+
+  /* Remove score columns from results and nvts if they were added in
+   *  migrations to previous versions.
+   */
+
+  sql ("DROP VIEW IF EXISTS vulns;");
+
+  sql ("ALTER TABLE nvts DROP COLUMN IF EXISTS score;");
+
+  sql ("ALTER TABLE results DROP COLUMN IF EXISTS score;");
+
+  sql ("ALTER TABLE results_trash DROP COLUMN IF EXISTS score;");
+
+  /* Change the vt_severities table to use a CVSS score
+   * if it already exists (migrating from a 21.04 development version) */
+  if (sql_int ("SELECT EXISTS (SELECT * FROM information_schema.tables"
+               "               WHERE table_catalog = '%s'"
+               "               AND table_schema = 'public'"
+               "               AND table_name = 'vt_severities')"
+               " ::integer;",
+               sql_database ()))
+    {
+      sql ("ALTER TABLE vt_severities ALTER COLUMN score"
+           " SET DATA TYPE double precision;");
+      sql ("UPDATE vt_severities"
+           " SET score = round((score / 10.0)::numeric, 1);");
+    }
+
+  /* Set the database version to 242. */
+
+  set_db_version (242);
+
+  sql_commit ();
+
+  return 0;
+}
+
 #undef UPDATE_DASHBOARD_SETTINGS
 
 /**
@@ -1657,6 +2678,27 @@ static migrator_t database_migrators[] = {
   {219, migrate_218_to_219},
   {220, migrate_219_to_220},
   {221, migrate_220_to_221},
+  {222, migrate_221_to_222},
+  {223, migrate_222_to_223},
+  {224, migrate_223_to_224},
+  {225, migrate_224_to_225},
+  {226, migrate_225_to_226},
+  {227, migrate_226_to_227},
+  {228, migrate_227_to_228},
+  {229, migrate_228_to_229},
+  {230, migrate_229_to_230},
+  {231, migrate_230_to_231},
+  {232, migrate_231_to_232},
+  {233, migrate_232_to_233},
+  {234, migrate_233_to_234},
+  {235, migrate_234_to_235},
+  {236, migrate_235_to_236},
+  {237, migrate_236_to_237},
+  {238, migrate_237_to_238},
+  {239, migrate_238_to_239},
+  {240, migrate_239_to_240},
+  {241, migrate_240_to_241},
+  {242, migrate_241_to_242},
   /* End marker. */
   {-1, NULL}};
 
@@ -1669,7 +2711,8 @@ static migrator_t database_migrators[] = {
  * @return TRUE if yes, else FALSE.
  */
 gboolean
-manage_migrate_needs_timezone (GSList *log_config, const gchar *database)
+manage_migrate_needs_timezone (GSList *log_config,
+                               const db_conn_info_t *database)
 {
   int db_version;
   g_log_set_handler (
@@ -1721,7 +2764,7 @@ migrate_is_available (int old_version, int new_version)
  * -1 error, -11 error running SCAP migration, -12 error running CERT migration.
  */
 int
-manage_migrate (GSList *log_config, const gchar *database)
+manage_migrate (GSList *log_config, const db_conn_info_t *database)
 {
   migrator_t *migrators;
   /* The version on the disk. */
@@ -1747,7 +2790,7 @@ manage_migrate (GSList *log_config, const gchar *database)
   if (old_version == -2)
     {
       g_warning ("%s: no task tables yet, so no need to migrate them",
-                 __FUNCTION__);
+                 __func__);
       version_current = 1;
     }
   else if (old_version == new_version)
@@ -1883,6 +2926,7 @@ manage_migrate (GSList *log_config, const gchar *database)
    * Reopen the database before the ANALYZE, in case the schema has changed. */
   cleanup_manage_process (TRUE);
   init_manage_process (database);
+  g_info ("   Analyzing the database. This may take up to several hours.");
   sql ("ANALYZE;");
 
   cleanup_manage_process (TRUE);
