@@ -1,20 +1,19 @@
-/* Copyright (C) 2014-2018 Greenbone Networks GmbH
+/* Copyright (C) 2014-2020 Greenbone Networks GmbH
  *
- * SPDX-License-Identifier: GPL-2.0-or-later
+ * SPDX-License-Identifier: AGPL-3.0-or-later
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -36,8 +35,6 @@
 #define SEVERITY_LOG 0.0
 // False positive severity constant
 #define SEVERITY_FP -1.0
-// Debug message severity constant
-#define SEVERITY_DEBUG -2.0
 // Error message severity constant
 #define SEVERITY_ERROR -3.0
 // Constant for missing or invalid severity
@@ -62,10 +59,10 @@ int
 manage_count_hosts_max (const char *, const char *, int);
 
 double
-level_min_severity (const char *, const char *);
+level_min_severity (const char *);
 
 double
-level_max_severity (const char *, const char *);
+level_max_severity (const char *);
 
 int
 valid_db_resource_type (const char *);
@@ -76,11 +73,14 @@ hosts_str_contains (const char *, const char *, int);
 void
 blank_control_chars (char *);
 
+icaltimezone *
+icalendar_timezone_from_string (const char *);
+
 icalcomponent *
 icalendar_from_old_schedule_data (time_t, time_t, time_t, time_t, int);
 
 icalcomponent *
-icalendar_from_string (const char *, gchar **);
+icalendar_from_string (const char *, icaltimezone *, gchar **);
 
 int
 icalendar_approximate_rrule_from_vcalendar (icalcomponent *, time_t *, time_t *,
@@ -96,6 +96,9 @@ int
 icalendar_duration_from_vcalendar (icalcomponent *);
 
 time_t
-icalendar_first_time_from_vcalendar (icalcomponent *, const char *);
+icalendar_first_time_from_vcalendar (icalcomponent *, icaltimezone *);
+
+gchar *
+clean_hosts_string (const char *);
 
 #endif /* not _GVMD_MANAGE_UTILS_H */
