@@ -1,20 +1,19 @@
-/* Copyright (C) 2012-2018 Greenbone Networks GmbH
+/* Copyright (C) 2012-2021 Greenbone Networks GmbH
  *
- * SPDX-License-Identifier: GPL-2.0-or-later
+ * SPDX-License-Identifier: AGPL-3.0-or-later
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -26,6 +25,7 @@
 #define _GVMD_SQL_H
 
 #include "iterator.h"
+#include "manage.h"
 
 #include <glib.h>
 
@@ -59,7 +59,7 @@ int
 sql_is_open ();
 
 int
-sql_open (const char *);
+sql_open (const db_conn_info_t *);
 
 void
 sql_close ();
@@ -79,15 +79,15 @@ sql_nquote (const char *, size_t);
 gchar *
 sql_quote (const char *);
 
+gchar *
+sql_ascii_escape_and_quote (const char *);
+
 gchar *
 sql_insert (const char *);
 
 void
 sql (char *sql, ...);
 
-void
-sqli (resource_t *, char *, ...);
-
 int
 sql_error (char *sql, ...);
 
@@ -115,6 +115,9 @@ sql_int64_0 (char *sql, ...);
 void
 sql_rename_column (const char *, const char *, const char *, const char *);
 
+int
+sql_cancel_internal ();
+
 /* Transactions. */
 
 void
@@ -135,10 +138,10 @@ sql_rollback ();
  * that is allowed to run SQL statements directly. */
 
 void
-init_prepared_iterator (iterator_t *, sql_stmt_t *);
+init_iterator (iterator_t *, const char *, ...);
 
 void
-init_iterator (iterator_t *, const char *, ...);
+iterator_rewind (iterator_t *iterator);
 
 double
 iterator_double (iterator_t *, int);
@@ -155,51 +158,13 @@ iterator_null (iterator_t *, int);
 const char *
 iterator_string (iterator_t *, int);
 
+gchar **
+iterator_array (iterator_t *, int);
+
 const char *
 iterator_column_name (iterator_t *, int);
 
 int
 iterator_column_count (iterator_t *);
 
-/* Prepared statements. */
-
-sql_stmt_t *
-sql_prepare (const char *sql, ...);
-
-int
-sql_bind_blob (sql_stmt_t *, int, const void *, int);
-
-int
-sql_bind_int64 (sql_stmt_t *, int, long long int *);
-
-int
-sql_bind_text (sql_stmt_t *, int, const gchar *, gsize);
-
-int
-sql_bind_double (sql_stmt_t *, int, double *);
-
-int
-sql_exec (sql_stmt_t *);
-
-void
-sql_finalize (sql_stmt_t *);
-
-int
-sql_reset (sql_stmt_t *);
-
-double
-sql_column_double (sql_stmt_t *, int);
-
-const char *
-sql_column_text (sql_stmt_t *, int);
-
-int
-sql_column_int (sql_stmt_t *, int);
-
-long long int
-sql_column_int64 (sql_stmt_t *, int);
-
-int
-sql_cancel_internal ();
-
 #endif /* not _GVMD_SQL_H */
