@@ -1,20 +1,19 @@
-/* Copyright (C) 2014-2018 Greenbone Networks GmbH
+/* Copyright (C) 2014-2021 Greenbone Networks GmbH
  *
- * SPDX-License-Identifier: GPL-2.0-or-later
+ * SPDX-License-Identifier: AGPL-3.0-or-later
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU Affero General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -124,30 +123,13 @@ sql_select_limit (int max)
     return "ALL";
   if (snprintf (string, 19, "%i", max) < 0)
     {
-      g_warning ("%s: snprintf failed", __FUNCTION__);
+      g_warning ("%s: snprintf failed", __func__);
       abort ();
     }
   string[19] = '\0';
   return string;
 }
 
-/**
- * @brief Add param to statement.
- *
- * @param[in]  stmt          Statement.
- * @param[in]  param_value   Value.
- * @param[in]  param_size    Size.
- * @param[in]  param_format  0 text, 1 binary.
- */
-static void
-sql_stmt_param_add (sql_stmt_t *stmt, const char *param_value,
-                    int param_size, int param_format)
-{
-  array_add (stmt->param_values, g_strndup (param_value, param_size));
-  g_array_append_val (stmt->param_lengths, param_size);
-  g_array_append_val (stmt->param_formats, param_format);
-}
-
 /**
  * @brief Init statement, preserving SQL.
  *
@@ -196,18 +178,67 @@ sql_is_open ()
   return conn ? 1 : 0;
 }
 
+#ifndef NDEBUG
+#include <execinfo.h>
+
+/**
+ * @brief Maximum number of frames in backtrace.
+ *
+ * For debugging backtrace in \ref log_notice.
+ */
+#define BA_SIZE 100
+#endif
+
 /**
  * @brief Log a NOTICE message.
  *
- * @param[in]  arg      Dummy arg.
- * @param[in]  message  Arg.
- *
- * @return 0 success, -1 error.
+ * @param[in]  arg     Dummy arg.
+ * @param[in]  result  Arg.
  */
 static void
-log_notice (void *arg, const char *message)
+log_notice (void *arg, const PGresult *result)
 {
-  g_debug ("%s", message);
+  g_debug ("PQ notice: %s", PQresultErrorMessage (result));
+
+#ifndef NDEBUG
+  char *verbose;
+
+  verbose = PQresultVerboseErrorMessage (result, PQERRORS_VERBOSE, PQSHOW_CONTEXT_ALWAYS);
+  g_debug ("PQ notice: verbose: %s", verbose);
+  PQfreemem (verbose);
+#endif
+
+  g_debug ("PQ notice: detail: %s",
+           PQresultErrorField (result, PG_DIAG_MESSAGE_DETAIL));
+  g_debug ("PQ notice: hint: %s",
+           PQresultErrorField (result, PG_DIAG_MESSAGE_HINT));
+  g_debug ("PQ notice:     table %s.%s",
+           PQresultErrorField (result, PG_DIAG_SCHEMA_NAME),
+           PQresultErrorField (result, PG_DIAG_TABLE_NAME));
+  g_debug ("PQ notice:     from %s in %s:%s",
+           PQresultErrorField (result, PG_DIAG_SOURCE_FUNCTION),
+           PQresultErrorField (result, PG_DIAG_SOURCE_FILE),
+           PQresultErrorField (result, PG_DIAG_SOURCE_LINE));
+  g_debug ("PQ notice: context:\n%s",
+           PQresultErrorField (result, PG_DIAG_CONTEXT));
+
+#ifndef NDEBUG
+  void *frames[BA_SIZE];
+  int frame_count, index;
+  char **frames_text;
+
+  /* Print a backtrace. */
+  frame_count = backtrace (frames, BA_SIZE);
+  frames_text = backtrace_symbols (frames, frame_count);
+  if (frames_text == NULL)
+    {
+      perror ("backtrace symbols");
+      frame_count = 0;
+    }
+  for (index = 0; index < frame_count; index++)
+    g_debug ("%s", frames_text[index]);
+  free (frames_text);
+#endif
 }
 
 /**
@@ -232,17 +263,6 @@ sql_default_database ()
   return "gvmd";
 }
 
-/**
- * @brief Turn off recursive triggers.
- *
- * Ignored when DB is Postgres.
- */
-void
-sql_recursive_triggers_off ()
-{
-  return;
-}
-
 /**
  * @brief Open the database.
  *
@@ -251,30 +271,37 @@ sql_recursive_triggers_off ()
  * @return 0 success, -1 error.
  */
 int
-sql_open (const char *database)
+sql_open (const db_conn_info_t *database)
 {
   gchar *conn_info;
   PostgresPollingStatusType poll_status;
   int socket;
 
-  conn_info = g_strdup_printf ("dbname='%s' application_name='%s'",
-                               database
-                                ? database
+  conn_info = g_strdup_printf ("dbname='%s'"
+                               " host='%s'"
+                               " port='%s'"
+                               " user='%s'"
+                               " application_name='%s'",
+                               database->name
+                                ? database->name
                                 : sql_default_database (),
+                               database->host ? database->host : "",
+                               database->port ? database->port : "",
+                               database->user ? database->user : "",
                                "gvmd");
   conn = PQconnectStart (conn_info);
   g_free (conn_info);
   if (conn == NULL)
     {
       g_warning ("%s: PQconnectStart failed to allocate conn",
-                 __FUNCTION__);
+                 __func__);
       return -1;
     }
   if (PQstatus (conn) == CONNECTION_BAD)
     {
       g_warning ("%s: PQconnectStart to '%s' failed: %s",
-                 __FUNCTION__,
-                 database ? database : sql_default_database (),
+                 __func__,
+                 database->name ? database->name : sql_default_database (),
                  PQerrorMessage (conn));
       goto fail;
     }
@@ -282,13 +309,13 @@ sql_open (const char *database)
   socket = PQsocket (conn);
   if (socket == 0)
     {
-      g_warning ("%s: PQsocket 0", __FUNCTION__);
+      g_warning ("%s: PQsocket 0", __func__);
       goto fail;
     }
 
   poll_status = PGRES_POLLING_WRITING;
 
-  g_debug ("%s: polling", __FUNCTION__);
+  g_debug ("%s: polling", __func__);
 
   while (1)
     {
@@ -306,7 +333,7 @@ sql_open (const char *database)
           if (ret < 0)
             {
               g_warning ("%s: write select failed: %s",
-                         __FUNCTION__, strerror (errno));
+                         __func__, strerror (errno));
               goto fail;
             }
           /* Poll again. */
@@ -325,7 +352,7 @@ sql_open (const char *database)
           if (ret < 0)
             {
               g_warning ("%s: read select failed: %s",
-                         __FUNCTION__, strerror (errno));
+                         __func__, strerror (errno));
               goto fail;
             }
           /* Poll again. */
@@ -333,8 +360,8 @@ sql_open (const char *database)
       else if (poll_status == PGRES_POLLING_FAILED)
         {
           g_warning ("%s: PQconnectPoll failed",
-                     __FUNCTION__);
-          g_warning ("%s: PQerrorMessage (conn): %s", __FUNCTION__,
+                     __func__);
+          g_warning ("%s: PQerrorMessage (conn): %s", __func__,
                      PQerrorMessage (conn));
           goto fail;
         }
@@ -345,20 +372,20 @@ sql_open (const char *database)
       poll_status = PQconnectPoll (conn);
     }
 
-  PQsetNoticeProcessor (conn, log_notice, NULL);
+  PQsetNoticeReceiver (conn, log_notice, NULL);
 
-  g_debug ("%s:   db: %s", __FUNCTION__, PQdb (conn));
-  g_debug ("%s: user: %s", __FUNCTION__, PQuser (conn));
-  g_debug ("%s: host: %s", __FUNCTION__, PQhost (conn));
-  g_debug ("%s: port: %s", __FUNCTION__, PQport (conn));
-  g_debug ("%s: socket: %i", __FUNCTION__, PQsocket (conn));
-  g_debug ("%s: postgres version: %i", __FUNCTION__, PQserverVersion (conn));
+  g_debug ("%s:   db: %s", __func__, PQdb (conn));
+  g_debug ("%s: user: %s", __func__, PQuser (conn));
+  g_debug ("%s: host: %s", __func__, PQhost (conn));
+  g_debug ("%s: port: %s", __func__, PQport (conn));
+  g_debug ("%s: socket: %i", __func__, PQsocket (conn));
+  g_debug ("%s: postgres version: %i", __func__, PQserverVersion (conn));
 
   if (PQserverVersion (conn) < 90600)
     {
       g_warning ("%s: PostgreSQL version 9.6 (90600) or higher is required",
-                 __FUNCTION__);
-      g_warning ("%s: Current version is %i", __FUNCTION__,
+                 __func__);
+      g_warning ("%s: Current version is %i", __func__,
                  PQserverVersion (conn));
       goto fail;
     }
@@ -387,7 +414,6 @@ sql_close ()
 void
 sql_close_fork ()
 {
-  // FIX PQfinish?
   conn = NULL;
 }
 
@@ -414,52 +440,6 @@ sql_last_insert_id ()
   return sql_int ("SELECT LASTVAL ();");
 }
 
-/**
- * @brief Perform an SQL statement, retrying if database is busy or locked.
- *
- * @param[out] resource  Last inserted resource.
- * @param[in]  sql       Format string for SQL statement.
- * @param[in]  ...       Arguments for format string.
- */
-void
-sqli (resource_t *resource, char* sql, ...)
-{
-  gchar *new_sql;
-  sql_stmt_t* stmt;
-  int sql_x_ret;
-  va_list args;
-
-  assert (sql && strlen (sql) && (sql[strlen (sql) - 1] != ';'));
-
-  /* Append RETURNING clause to SQL. */
-
-  new_sql = g_strdup_printf ("%s RETURNING id;", sql);
-
-  /* Run statement, returning integer. */
-
-  va_start (args, sql);
-  sql_x_ret = sql_x (new_sql, args, &stmt);
-  va_end (args);
-  g_free (new_sql);
-  switch (sql_x_ret)
-    {
-      case  0:
-        break;
-      case  1:
-        sql_finalize (stmt);
-        abort ();
-      default:
-        assert (0);
-        /* Fall through. */
-      case -1:
-        sql_finalize (stmt);
-        abort ();
-    }
-  if (resource)
-    *resource = sql_column_int64 (stmt, 0);
-  sql_finalize (stmt);
-}
-
 /**
  * @brief Prepare a statement.
  *
@@ -488,7 +468,7 @@ sql_prepare_internal (int retry, int log, const char* sql, va_list args,
 }
 
 /**
- * @brief Execute a prepared statement.
+ * @brief Execute a statement.
  *
  * @param[in]  retry  Whether to keep retrying while database is busy or locked.
  * @param[in]  stmt   Statement.
@@ -505,8 +485,6 @@ sql_exec_internal (int retry, sql_stmt_t *stmt)
 
   if (stmt->executed == 0)
     {
-      // FIX retry?
-
       result = PQexecParams (conn,
                              stmt->sql,
                              stmt->param_values->len,
@@ -521,18 +499,18 @@ sql_exec_internal (int retry, sql_stmt_t *stmt)
           char *sqlstate;
 
           sqlstate = PQresultErrorField (result, PG_DIAG_SQLSTATE);
-          g_debug ("%s: sqlstate: %s", __FUNCTION__, sqlstate);
+          g_debug ("%s: sqlstate: %s", __func__, sqlstate);
           if (sqlstate && (strcmp (sqlstate, "57014") == 0))
             {
               /* query_canceled */
               log_errors = 0;
-              g_debug ("%s: canceled SQL: %s", __FUNCTION__, stmt->sql);
+              g_debug ("%s: canceled SQL: %s", __func__, stmt->sql);
             }
           else if (sqlstate && (strcmp (sqlstate, "55P03") == 0))
             {
               /* lock_not_available */
               g_debug ("%s: lock unavailable: %s",
-                       __FUNCTION__,
+                       __func__,
                        PQresultErrorMessage(result));
               return -3;
             }
@@ -540,25 +518,28 @@ sql_exec_internal (int retry, sql_stmt_t *stmt)
             {
               /* unique_violation */
               g_warning ("%s: constraint violation: %s",
-                         __FUNCTION__,
+                         __func__,
                          PQresultErrorMessage (result));
-              g_warning ("%s: SQL: %s", __FUNCTION__, stmt->sql);
+              g_warning ("%s: SQL: %s", __func__, stmt->sql);
               return -4;
             }
-
+            else if (sqlstate && (strcmp (sqlstate, "40P01") == 0))
+              {
+                /* deadlock_detected */
+                g_debug ("%s: deadlock: %s",
+                         __func__,
+                         PQresultErrorMessage (result));
+                g_debug ("%s: SQL: %s", __func__, stmt->sql);
+                return -5;
+              }
           if (log_errors)
             {
               g_warning ("%s: PQexec failed: %s (%i)",
-                         __FUNCTION__,
+                         __func__,
                          PQresultErrorMessage (result),
                          PQresultStatus (result));
-              g_warning ("%s: SQL: %s", __FUNCTION__, stmt->sql);
+              g_warning ("%s: SQL: %s", __func__, stmt->sql);
             }
-#if 0
-          // FIX ?
-          PQclear (result);
-          PQfinish (conn);
-#endif
           return -1;
         }
 
@@ -575,55 +556,6 @@ sql_exec_internal (int retry, sql_stmt_t *stmt)
   return 0;
 }
 
-/**
- * @brief Write debug messages with the query plan for an SQL query to the log.
- *
- * @param[in] sql   Format string for the SQL query.
- * @param[in] args  Format string arguments in a va_list.
- *
- * @return 0 success, -1 error.
- */
-int
-sql_explain_internal (const char* sql, va_list args)
-{
-  char *explain_sql;
-  sql_stmt_t *explain_stmt;
-  int explain_ret;
-
-  explain_sql = g_strconcat ("EXPLAIN ", sql, NULL);
-  if (sql_prepare_internal (1, 1, explain_sql, args, &explain_stmt))
-    {
-      if (log_errors)
-        g_warning ("%s : Failed to prepare EXPLAIN statement", __FUNCTION__);
-      g_free (explain_sql);
-      return -1;
-    }
-
-  while (1)
-    {
-      explain_ret = sql_exec_internal (1, explain_stmt);
-      if (explain_ret == 1)
-        g_debug ("%s : %s",
-                __FUNCTION__,
-                PQgetvalue (explain_stmt->result,
-                            explain_stmt->current_row,
-                            0));
-      else if (explain_ret == 0)
-        break;
-      else
-        {
-          g_warning ("%s : Failed to get EXPLAIN row", __FUNCTION__);
-          sql_finalize (explain_stmt);
-          g_free (explain_sql);
-          return -1;
-        }
-    }
-
-  sql_finalize (explain_stmt);
-  g_free (explain_sql);
-  return 0;
-}
-
 
 /* Transactions. */
 
@@ -686,78 +618,28 @@ iterator_null (iterator_t* iterator, int col)
 {
   if (iterator->done) abort ();
   assert (iterator->stmt->result);
-  return PQgetisnull (iterator->stmt->result, 0, col);
+  return PQgetisnull (iterator->stmt->result, iterator->stmt->current_row, col);
+}
+
+/**
+ * @brief Rewind an iterator to the beginning.
+ *
+ * This lets the caller iterate over the data again.
+ *
+ * @param[in]  iterator  Iterator.
+ */
+void
+iterator_rewind (iterator_t* iterator)
+{
+  iterator->done = FALSE;
+  iterator->stmt->current_row = -1;
 }
 
 
-/* Prepared statements. */
+/* Statements. */
 
 /**
- * @brief Bind a param to a statement.
- *
- * @param[in]  stmt          Statement.
- * @param[in]  position      Position in statement.
- * @param[in]  param_value   Param value.
- * @param[in]  param_size    Param size.
- * @param[in]  param_format  0 text, 1 binary.
- */
-static void
-bind_param (sql_stmt_t *stmt, int position, const void *param_value,
-            int param_size, int param_format)
-{
-  if (position > stmt->param_values->len + 1)
-    {
-      g_critical ("%s: binding out of order: parameter %i after %i",
-                  __FUNCTION__,
-                  position,
-                  stmt->param_values->len);
-      abort ();
-    }
-  sql_stmt_param_add (stmt, param_value, param_size, param_format);
-}
-
-/**
- * @brief Bind a blob to a statement.
- *
- * @param[in]  stmt        Statement.
- * @param[in]  position    Position in statement.
- * @param[in]  value       Blob.
- * @param[in]  value_size  Blob size.
- *
- * @return 0 success, -1 error.
- */
-int
-sql_bind_blob (sql_stmt_t *stmt, int position, const void *value,
-               int value_size)
-{
-  bind_param (stmt, position, value, value_size, 1);
-  return 0;
-}
-
-/**
- * @brief Bind a text value to a statement.
- *
- * @param[in]  stmt        Statement.
- * @param[in]  position    Position in statement.
- * @param[in]  value       Value.
- * @param[in]  value_size  Value size, or -1 to use strlen of value.
- *
- * @return 0 success, -1 error.
- */
-int
-sql_bind_text (sql_stmt_t *stmt, int position, const gchar *value,
-               gsize value_size)
-{
-  bind_param (stmt,
-              position,
-              value,
-              value_size == -1 ? strlen (value) : value_size,
-              0);
-  return 0;
-}
-
-/**
- * @brief Free a prepared statement.
+ * @brief Free a statement.
  *
  * @param[in]  stmt  Statement.
  */
@@ -773,30 +655,7 @@ sql_finalize (sql_stmt_t *stmt)
 }
 
 /**
- * @brief Reset a prepared statement.
- *
- * @param[in]  stmt  Statement.
- *
- * @return 0 success, -1 error.
- */
-int
-sql_reset (sql_stmt_t *stmt)
-{
-  gchar *sql;
-
-  PQclear (stmt->result);
-  array_free (stmt->param_values);
-  g_array_free (stmt->param_lengths, TRUE);
-  g_array_free (stmt->param_formats, TRUE);
-
-  sql = stmt->sql;
-  sql_stmt_init (stmt);
-  stmt->sql = sql;
-  return 0;
-}
-
-/**
- * @brief Return a column as a double from a prepared statement.
+ * @brief Return a column as a double from a statement.
  *
  * It's up to the caller to ensure that there is a row available.
  *
@@ -815,7 +674,7 @@ sql_column_double (sql_stmt_t *stmt, int position)
 }
 
 /**
- * @brief Return a column as text from a prepared statement.
+ * @brief Return a column as text from a statement.
  *
  * It's up to the caller to ensure that there is a row available.
  *
@@ -834,7 +693,7 @@ sql_column_text (sql_stmt_t *stmt, int position)
 }
 
 /**
- * @brief Return a column as an integer from a prepared statement.
+ * @brief Return a column as an integer from a statement.
  *
  * It's up to the caller to ensure that there is a row available.
  *
@@ -864,7 +723,7 @@ sql_column_int (sql_stmt_t *stmt, int position)
 }
 
 /**
- * @brief Return a column as an int64 from a prepared statement.
+ * @brief Return a column as an int64 from a statement.
  *
  * It's up to the caller to ensure that there is a row available.
  *
@@ -893,6 +752,68 @@ sql_column_int64 (sql_stmt_t *stmt, int position)
     }
 }
 
+/**
+ * @brief Return a column as text from a statement.
+ *
+ * It's up to the caller to ensure that there is a row available.
+ *
+ * @param[in]  stmt      Statement.
+ * @param[in]  position  Column position.
+ *
+ * @return Column value.  NULL if column is NULL.
+ */
+gchar **
+sql_column_array (sql_stmt_t *stmt, int position)
+{
+  const char *text;
+
+  if (PQgetisnull (stmt->result, stmt->current_row, position))
+    return NULL;
+
+  /* {DFN-CERT-2017-1238,DFN-CERT-2014-1366,DFN-CERT-2014-1354} */
+
+  text = (const char*) PQgetvalue (stmt->result, stmt->current_row, position);
+  if (text && text[0] == '{')
+    {
+      gchar **array, **point, **last;
+
+      if (text[1] == '}')
+        return (gchar **) g_malloc0 (sizeof (gchar *));
+
+      array = g_strsplit (text + 1, ",", 0);
+      point = last = array;
+      while (*point)
+        {
+          last = point;
+          point++;
+        }
+      if (*last)
+        {
+          gchar *last_element;
+
+          last_element = *last;
+          if (*last_element == '\0')
+            /* Weird, last element should always have a }. */
+            g_warning ("%s: last element missing closing }", __func__);
+          else
+            {
+              while (*last_element)
+                last_element++;
+              last_element--;
+              /* Clip the trailing }. */
+              *last_element = '\0';
+            }
+        }
+      return array;
+    }
+
+  /* This shouldn't happen. */
+  assert (0);
+  g_warning ("%s: array column not NULL and does not contain array",
+             __func__);
+  return NULL;
+}
+
 /**
  * @brief Cancels the current SQL statement.
  *
